-- get the addon namespace
local addon, ns = ...
local cfg = ns.cfg
local lib = _G["HEDD_lib"] or CreateFrame("Frame","HEDD_lib")
ns.lib = lib

lib.classes = {}
lib.classevents = {}

lib.inrange = function(spell)
	if cfg.spells[spell] then 
		if IsSpellInRange(cfg.spells[spell].name,"target")==1 then return true end
	else
		if IsSpellInRange(spell,"target")==1 then return true end
	end
	return nil
end

local onground
lib.GroundUpdate = function()
	onground=cfg.onGround
	if IsFalling() or IsFlying() then
		cfg.onGround=nil
	else
		cfg.onGround=true
	end
	
	if onground~=cfg.onGround then
		cfg.Update=true
		return true
	end
	return nil
end

local shape
lib.UpdateShape = function()
	shape=cfg.shape
	cfg.shapeid=GetShapeshiftFormID()
	if cfg.shapeid and cfg.shapes[cfg.shapeid] then
		cfg.shape=cfg.shapes[cfg.shapeid]
	else 
		cfg.shape="human"
	end
	if shape~=cfg.shape then
		cfg.Update=true
		return true
	end
	return nil
end

local itemID,i
lib.UpdateSet = function()
	cfg.setitems={}
	for itemSET, _ in pairs(cfg.set) do
		cfg.setitems[itemSET]=0
		for _, itemID in pairs(cfg.set[itemSET]) do
			for i=0,19 do
				if cfg.equip[i] and cfg.equip[i]==itemID then
					cfg.setitems[itemSET]=cfg.setitems[itemSET]+1
				end
			end
		end
	end
end

lib.trinket = function(trinket)
	for _, itemID in pairs(cfg.trinket[trinket]) do
		for i=13,14 do
			if cfg.equip[i] and cfg.equip[i]==itemID then
				return true
			end
		end
	end
	return nil
end

lib.SetBonus=function(set)
	if cfg.setitems[set] then
		if cfg.setitems[set]>=4 then
			return 2
		elseif cfg.setitems[set]>=2 then
			return 1
		end
	end
	return 0
end

lib.UpdateEquip=function(slot)
	cfg.equip[slot]=GetInventoryItemID("player", slot)
end

lib.UpdateEquipAll=function()
	for i=0,19 do
		lib.UpdateEquip(i)
	end
	lib.UpdateSet()
end

local cspell
lib.SwitchMode = function()
	if cfg.mode~="dps" then
		for _,cspell in ipairs(cfg.spells_single) do
			if cfg.spells[cspell] and (cfg.lastspell==cfg.spells[cspell].id or lib.SpellCasting(cspell)) then
				cfg.mode = "dps"
				cfg.plist=cfg.plistdps
				cfg.Update=true
				return
			end
		end
	end
	
	if cfg.mode~="aoe" then
		for _,cspell in ipairs(cfg.spells_aoe) do
			if cfg.spells[cspell] and (cfg.lastspell==cfg.spells[cspell].id or lib.SpellCasting(cspell)) then
				cfg.mode = "aoe"
				cfg.plist=cfg.plistaoe
				cfg.Update=true
				return
			end
		end
	end
	
	if cfg.mode~="range" then
		for _,cspell in ipairs(cfg.spells_range) do
			if cfg.spells[cspell] and (cfg.lastspell==cfg.spells[cspell].id or lib.SpellCasting(cspell)) then
				cfg.mode = "range"
				cfg.plist=cfg.plistrange
				cfg.Update=true
				return
			end
		end
	end
end

lib.instance=function()
	cfg.instance=select(2,IsInInstance())
	--[[
	arena - Player versus player arena
	none - Not inside an instance
	party - 5-man instance
	pvp - Player versus player battleground
	raid - Raid instance
	]]
	return cfg.instance
end

lib.ininstance=function()
	if cfg.instance=="raid" or cfg.instance=="party" then
		return true 
	else
		return nil
	end
end

lib.raidboss = function()
	if cfg.target=="worldboss" and cfg.instance=="raid" then return true end
	return nil
end

lib.UpdateTarget = function()
	cfg.target=UnitClassification("target")
	if UnitLevel("target")==-1 then cfg.target="worldboss" end
	cfg.targetGUID=UnitGUID("target")
	--[[
	elite - Elite
	normal - Normal
	minus - Small
	rare - Rare
	rareelite - Rare-Elite
	worldboss - World Boss
	]]
--	print(cfg.target.." "..cfg.instance)
	return cfg.target
end

lib.hardtarget = function()
	if cfg.target=="normal" or cfg.target=="minus" then return nil end
	return true
end

lib.bosstarget = function()
	if cfg.target=="worldboss" then
		return true
	end
	return nil
end

cfg.power_tmp = {}
cfg.power_tmp.power=nil
cfg.power_tmp.powermax=nil
cfg.power_tmp.pactiveRegen=nil
cfg.power_tmp.realpower=nil

local power_save = function()
	cfg.power_tmp.power=cfg.power
	cfg.power_tmp.powermax=cfg.powermax
	cfg.power_tmp.pactiveRegen=cfg.pactiveRegen
	cfg.power_tmp.realpower=cfg.realpower
end

local power_changed = function()
	if cfg.power~=cfg.power_tmp.power or cfg.powermax~=cfg.power_tmp.powermax or cfg.pactiveRegen~=cfg.power_tmp.pactiveRegen or cfg.realpower~=cfg.power_tmp.realpower then
		return true
	else
		return false
	end
end

lib.UpdatePower = function(powerType)
	powerType=powerType or cfg.powertype
	if cfg.powertype~=powerType then
		lib.UpdateAltPower(powerType)
		return
	end
	power_save()
	cfg.power=UnitPower("player",powerType)
	cfg.powermax=UnitPowerMax("player",powerType)
	_,cfg.pactiveRegen=GetPowerRegen()
	cfg.realpower=cfg.power+lib.CastingPower()
	if cfg.realpower>cfg.powermax then cfg.realpower=cfg.powermax end 
	
	if power_changed() then
		cfg.Update=true
	end
end

local castpower
lib.CastingPower = function()
	if lib.IsCasting() then
		return ((cfg.Casting.castend-cfg.now)*cfg.pactiveRegen-lib.GetCastingSpellCost())
	end
	return 0
end

lib.Power = function()
	return cfg.realpower
end

lib.PowerMax = function()
	return cfg.powermax
end

lib.SetAltPower = function(altpower_num,nocombo)
	if not altpower_num or cfg.powertype_num==altpower_num then
		return
	end
	cfg.altpower_num=altpower_num
	cfg.altpower=UnitPower("player",cfg.altpower_num)
	cfg.altpowermax=UnitPowerMax("player",cfg.altpower_num)
	if not nocombo then
		lib.ComboAdd(Heddframe,cfg.altpowermax)
		Heddframe.combo.altpower=true
		--[[function Heddclassevents.UNIT_POWER_FREQUENT(unit,powerType)
			if unit=="player" and cfg.altpower_num==_G["SPELL_POWER_"..powerType] then
				lib.ComboSet(UnitPower("player", cfg.altpower_num))
			end
		end]]
	end
	cfg.Update=true
end


local altpower_save = function()
	cfg.power_tmp.power=cfg.altpower
	cfg.power_tmp.powermax=cfg.altpowermax
end

local altpower_changed = function()
	if cfg.altpower~=cfg.power_tmp.power or cfg.altpowermax~=cfg.power_tmp.powermax then
		return true
	else
		return false
	end
end

lib.UpdateAltPower = function(powerType)
	if cfg.altpower_num==cfg.powertype_num or cfg.altpower_num~=_G["SPELL_POWER_"..powerType] 
	then return end
	altpower_save()
	cfg.altpower = UnitPower("player", cfg.altpower_num)
	cfg.altpowermax=UnitPowerMax("player", cfg.altpower_num)
	if altpower_changed() then
		if Heddframe.combo.altpower then
			lib.ComboSet(cfg.altpower)
		end
		cfg.Update=true
	end
end

cfg.health_tmp = {}
cfg.health_tmp.health=nil
cfg.health_tmp.healthmax=nil
local health_save = function()
	cfg.health_tmp.health=cfg.health
	cfg.health_tmp.healthmax=cfg.healthmax
end

local health_changed = function()
	if cfg.health~=cfg.health_tmp.health or  cfg.healthmax~=cfg.health_tmp.healthmax then
		return true
	else
		return false
	end
end

lib.UpdateHealth = function()
	health_save()
	cfg.healthmax=UnitHealthMax("target")
	cfg.health=(cfg.healthmax==0) and 100 or hedlib.round(UnitHealth("target")*100/cfg.healthmax,0)
	if health_changed() then
		cfg.Update=true
		return true
	end
end

local playerhealth,playerhealthmax,playerhealthperc
lib.UpdatePlayerHealth = function()
	playerhealth=cfg.phealth
	playerhealthmax=cfg.phealthmax
	playerhealthperc=cfg.phealthperc
	cfg.phealthmax=UnitHealthMax("player")
	cfg.phealth=UnitHealth("player")
	cfg.phealthperc=(cfg.phealthmax==0) and 100 or hedlib.round(cfg.phealth*100/cfg.phealthmax,0)
	if cfg.phealth~=playerhealth or cfg.phealthmax~=playerhealthmax or cfg.phealthperc~=playerhealthperc then
		cfg.Update=true
	end
end

lib.UpdateHaste = function()
	cfg.haste=GetCombatRatingBonus(CR_HASTE_MELEE)
end

lib.MaxEnergy = function()
	return (cfg.powermax-cfg.realpower)/cfg.pactiveRegen
end

--[[lib.TimeEnergy = function(t)
	return (cfg.realpower+cfg.pactiveRegen*t)
end]]

lib.Time2Power = function(power)
	if cfg.realpower<power then
		return((power-cfg.realpower)/cfg.pactiveRegen)
	end
	return 0
end

lib.TimeEnergyGain = function(t)
	return (cfg.pactiveRegen*t)
end

local pcombo
lib.ComboUpdate = function()
	pcombo = cfg.combo
	cfg.combo = GetComboPoints('player', 'target')
	if pcombo~=cfg.combo then
		cfg.Update=true
		return true
	end
	return nil
end

local enabled, glyphType, glyphTooltipIndex, glyphSpell, icon
lib.HasGlyph = function(id)
	for i=1, NUM_GLYPH_SLOTS do
		enabled, glyphType, glyphTooltipIndex, glyphSpell, icon = GetGlyphSocketInfo(i)
		if glyphSpell==id then return true end
	end
	return nil
end

local region,text
lib.PrintTooltipLines_helper = function(...)
    for i = 1, select("#", ...) do
        region = select(i, ...)
        if region and region:GetObjectType() == "FontString" then
            text = region:GetText() or " "
			print(region:GetName().." "..text)
        end
    end
end

lib.PrintTooltipLines = function(tooltip)
    lib.PrintTooltipLines_helper(tooltip:GetRegions())
end

lib.PrintSpellTooltip = function(spellid)
	local Hedd_Tooltip = _G["Hedd_Tooltip"] or CreateFrame('GameTooltip', 'Hedd_Tooltip', nil, 'GameTooltipTemplate')
	Hedd_Tooltip:SetOwner(UIParent, 'ANCHOR_NONE')
	Hedd_Tooltip:SetSpellByID(spellid)
	lib.PrintTooltipLines(Hedd_Tooltip)
	Hedd_Tooltip:Hide()
end

lib.IsTextInSpellTooltip = function(spellid,txt)
	Heddframe.Hedd_Tooltip:SetOwner(UIParent, 'ANCHOR_NONE')
	Heddframe.Hedd_Tooltip:SetSpellByID(spellid)
	txt = hedlib.IsTextInTooltip(Heddframe.Hedd_Tooltip,txt)
	Heddframe.Hedd_Tooltip:Hide()
	return(txt)
end